#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//    starz5bo.cpp - bolometric corrections for starz4.cpp.
//
//
//    - PROJECT STARMAN -  Programmer: James A. Green.
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz4.cpp - main module.
//		      starz4ag.cpp - Stellar ages and evolution, perturbations.
//		      starz4au.cpp - Neutrino luminosity & stellar wind.
//		      starz4fu.cpp - basic functions.
//		      starz4bo.cpp - bolometric corrections.
//		      starz4op.cpp - supplemental ops.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//
//     GLOBAL VARIABLES - prototypes - visible as externals in starz4.cpp
//
double bolometric(double xt);
void bcprobe(double bc, int i);
//
//     EXTERNAL VARIABLES - defined in starz4.cpp
//
extern int bcflag;     // 0 -> Zombeck bolometric corrections.
//                        1 -> Schwarzschild bolometric corrections.
extern char value[50]; // input value string.
extern int d;          // input value.
//
/************************************************************
*
*    bolometric(xt) - Return bolometric correction from
*                     xt = log10(Tsurface).
*
************************************************************/
double bolometric(double xt)
{
       double bc;
//
// bcflag == 0->Use Zombeck [9] 1990 values.
//   == 1->Use Schwarzschild [8] values.
//
// bcflag is 0, because STARZ elimated the Schwarzschild table.
//
// need to insert the bolometric correction,
//  a function of the spectrum.
//
//  L/Lsun
// = 10**{[4.8 + BCsun - [m + 5 - 5*log10(r) + BCstar(Tsurface)]]/2.5}
// = 10**{[4.63 - [m + 5 - 5*log10(r) + BCstar(Tsurface)]]/2.5}
//
// M.Schwarzschild B.C. (1950s)  Martin V. Zombeck (1990) values
// STRUCTURE & EVOLUTION OF THE  HANDBOOK OF SPACE ASTRONOMY &
// STARS, pg.9.                  ASTROPHYSICS [9], derived from p.68-69.
// (after Balloon measurements?) (after satellite measurements?) Origin:
//                               Lang, K.,Astrophysical Formulae,
//                               Springer Verlag,1974.
//
//  log10(Tsurface)    BCstar              log10(Tsurface)   BCstar
//	4.65		-4.1               4.5               -3.34
//	4.46		-3.0               4.48              -3.17
//	4.32		-2.3               4.43              -2.8
//	4.21		-1.7               4.38              -2.5
//	4.13		-1.25              4.34              -2.23
//	4.08		-0.95              4.27              -1.77
//	4.03		-0.72              4.21              -1.39
//	3.99		-0.52              4.18              -1.21
//	3.95		-0.38              4.16              -1.04
//	3.91		-0.2               4.13              -0.85
//	3.87		-0.09              4.09              -0.66
//	3.83		-0.02              4.03              -0.4
//	3.8		-0.0               3.98              -0.25
//	3.76		-0.03              3.935             -0.15
//	3.73		-0.09              3.913             -0.12
//	3.69		-0.18              3.86              -0.08
//	3.66	        -0.29              3.84              -0.06
//      3.63		-0.42              3.815             -0.04
//	3.61		-0.57              3.79              -0.05
//	3.58		-0.74              3.77              -0.06
//	3.56		-0.92              3.76              -0.07
//	3.55		-1.11              3.74              -0.10
//     	3.53		-1.3               3.73              -0.15
//	3.52		-1.5               3.72              -0.19
//	3.52		-1.7               3.68              -0.25
//	3.51		-1.9               3.64              -0.65
//                                         3.62              -0.9
//                                         3.59              -1.2
//                                         3.55              -1.48
//                                         3.54              -1.76
//                                         3.53              -2.03
//                                         3.51              -2.31
//                                         3.49              -2.62
//                                         3.42              -4.2
//   
//
    if(bcflag == 1){ //Choose between Schwarzschild [8]
//                     & Zombeck [9] scheme corrections.
	if(xt > 4.65) //  1->Schwarzschild [8] B.C. corrections,1950s.
	   bc = -4.1; //  install estimates for ends of range.
	if(xt < 3.51) // bcflag is a compile-time option.
	   bc = -1.9;
	if(xt <= 4.65 && xt >= 4.46){
	   bc = (4.65 - xt)*((4.1 - 3.0)/(4.65 - 4.46)) - 4.1;
	   bcprobe(bc,1);
	}
	if(xt <= 4.46 && xt >= 4.32){
	   bc = (4.46 - xt)*((3.0 - 2.3)/(4.46 - 4.32)) - 3.0;
	   bcprobe(bc,2);
	}
	if(xt <= 4.32 && xt >= 4.21){
	   bc = (4.32 - xt)*((2.3 - 1.7)/(4.32 - 4.21)) - 2.3;
	   bcprobe(bc,3);
	}
	if(xt <= 4.21 && xt >= 4.13){
	   bc = (4.21 - xt)*((1.7 - 1.25)/(4.21 - 4.13)) - 1.7;
	   bcprobe(bc,4);
	}
	if(xt <= 4.13 && xt >= 4.08){
	   bc = (4.13 - xt)*((1.25 - 0.95)/(4.13 - 4.08)) - 1.25;
	   bcprobe(bc,5);
	}
	if(xt <= 4.08 && xt >= 4.03){
	   bc = (4.08 - xt)*((0.95 - 0.72)/(4.08 - 4.03)) - 0.95;
	   bcprobe(bc,6);
	}
	if(xt <= 4.03 && xt >= 3.99){
	   bc = (4.03 - xt)*((0.72 - 0.52)/(4.03 - 3.99)) - 0.72;
	   bcprobe(bc,7);
	}
	if(xt <= 3.99 && xt >= 3.95){
	   bc = (3.99 - xt)*((0.52 - 0.38)/(3.99 - 3.95)) - 0.52;
	   bcprobe(bc,1);
	}
	if(xt <= 3.95 && xt >= 3.91){
	   bc = (3.95 - xt)*((0.38 - 0.20)/(3.95 - 3.91)) - 0.38;
	   bcprobe(bc,8);
	}
	if(xt <= 3.91 && xt >= 3.87){
	   bc = (3.91 - xt)*((0.20 - 0.09)/(3.91 - 3.87)) - 0.20;
	   bcprobe(bc,9);
	}
	if(xt <= 3.87 && xt >= 3.83){
	   bc = (3.87 - xt)*((0.09 - 0.02)/(3.87 - 3.83)) - 0.09;
	   bcprobe(bc,10);
	}
	if(xt <= 3.83 && xt >= 3.8){
	   bc = (3.83 - xt)*((0.02 - 0.00)/(3.83 - 3.80)) - 0.02;
	   bcprobe(bc,11);
	}
	if(xt <= 3.8 && xt >= 3.76){
	   bc = (3.8 - xt)*((0.0 - 0.03)/(3.80 - 3.76)) - 0.0;
	   bcprobe(bc,12);
	}
	if(xt <= 3.76 && xt >= 3.73){
	   bc = (3.76 - xt)*((0.03 - 0.09)/(3.76 - 3.73)) - 0.03;
	   bcprobe(bc,13);
	}
	if(xt <= 3.73 && xt >= 3.69){
	   bc = (3.73 - xt)*((0.09 - 0.18)/(3.73 - 3.69)) - 0.09;
	   bcprobe(bc,14);
	}
	if(xt <= 3.69 && xt >= 3.66){
	   bc = (3.69 - xt)*((0.18 - 0.29)/(3.69 - 3.66)) - 0.18;
	   bcprobe(bc,15);
	}
	if(xt <= 3.66 && xt >= 3.63){
	   bc = (3.66 - xt)*((0.29 - 0.42)/(3.66 - 3.63)) - 0.29;
	   bcprobe(bc,16);
	}
	if(xt <= 3.63 && xt >= 3.61){
	   bc = (3.63 - xt)*((0.42 - 0.57)/(3.63 - 3.61)) - 0.42;
	   bcprobe(bc,17);
	}
	if(xt <= 3.61 && xt >= 3.58){
	   bc = (3.61 - xt)*((0.57 - 0.74)/(3.61 - 3.58)) - 0.57;
	   bcprobe(bc,18);
	}
	if(xt <= 3.58 && xt >= 3.56){
	   bc = (3.58 - xt)*((0.74 - 0.92)/(3.58 - 3.56)) - 0.74;
	   bcprobe(bc,19);
	}
	if(xt <= 3.56 && xt >= 3.55){
	   bc = (3.56 - xt)*((0.92 - 1.11)/(3.56 - 3.55)) - 0.92;
	   bcprobe(bc,20);
	}
	if(xt <= 3.55 && xt >= 3.53){
	   bc = (3.55 - xt)*((1.11 - 1.3)/(3.55 - 3.53)) - 1.11;
	   bcprobe(bc,21);
	}
	if(xt <= 3.53 && xt >= 3.52){
	   bc = (3.53 - xt)*((1.3 - 1.5)/(3.53 - 3.52)) - 1.3;
	   bcprobe(bc,22);
	}
	if(xt <= 3.52 && xt >= 3.51){
	   bc = (3.52 - xt)*((1.5 - 1.9)/(3.52 - 3.51)) - 1.5;
	   bcprobe(bc,23);
	}
	return(bc);
    }
    else if(bcflag == 0){  // Conditionally use
//                            0->Zombeck[9] table values, p68-69.
	if(xt > 4.5) //  Martin Zombeck [9] B.C. corrections, circa 1990.
	   bc = -3.34;  //  install estimates for ends of range.
	if(xt < 3.42)   // bcflag is a compile-time option.
	   bc = -4.2;
	if(xt <= 4.5 && xt >= 4.48){
	   bc = (4.5 - xt)*((3.34 - 3.17)/(4.5 - 4.48)) - 3.34;
           bcprobe(bc,1);
	}
	if(xt <= 4.48 && xt >= 4.43){
	   bc = (4.48 - xt)*((3.17 - 2.8)/(4.48 - 4.43)) - 3.17;
	   bcprobe(bc,2);
	}
	if(xt <= 4.43 && xt >= 4.38){
	   bc = (4.43 - xt)*((2.8 - 2.5)/(4.43 - 4.38)) - 2.8;
	   bcprobe(bc,3);
	}
	if(xt <= 4.38 && xt >= 4.34){
	   bc = (4.38 - xt)*((2.5 - 2.23)/(4.38 - 4.34)) - 2.5;
	   bcprobe(bc,4);
	}
	if(xt <= 4.34 && xt >= 4.27){
	   bc = (4.34 - xt)*((2.23 - 1.77)/(4.34 - 4.27)) - 2.23;
	   bcprobe(bc,5);
	}
	if(xt <= 4.27 && xt >= 4.21){
	   bc = (4.27 - xt)*((1.77 - 1.39)/(4.27 - 4.21)) - 1.77;
           bcprobe(bc,6);
	}
	if(xt <= 4.21 && xt >= 4.18){
	   bc = (4.21 - xt)*((1.39 - 1.21)/(4.21 - 4.18)) - 1.39;
           bcprobe(bc,7);
	}
	if(xt <= 4.18 && xt >= 4.16){
	   bc = (4.18 - xt)*((1.21 - 1.04)/(4.18 - 4.16)) - 1.21;
           bcprobe(bc,8);
	}
	if(xt <= 4.16 && xt >= 4.13){
	   bc = (4.16 - xt)*((1.04 - 0.85)/(4.16 - 4.13)) - 1.04;
           bcprobe(bc,9);
	}
	if(xt <= 4.13 && xt >= 4.09){
	   bc = (4.13 - xt)*((0.85 - 0.66)/(4.13 - 4.09)) - 0.85;
           bcprobe(bc,10);
	}
	if(xt <= 4.09 && xt >= 4.03){
	   bc = (4.09 - xt)*((0.66 - 0.4)/(4.09 - 4.03)) - 0.66;
           bcprobe(bc,11);
	}
	if(xt <= 4.03 && xt >= 3.98){
	   bc = (4.03 - xt)*((0.4 - 0.25)/(4.03 - 3.98)) - 0.4;
           bcprobe(bc,12);
	}
	if(xt <= 3.98 && xt >= 3.935){
	   bc = (3.98 - xt)*((0.25 - 0.15)/(3.98 - 3.935)) - 0.25;
           bcprobe(bc,13);
	}
	if(xt <= 3.935 && xt >= 3.913){
	   bc = (3.935 - xt)*((0.15 - 0.12)/(3.935 - 3.913)) - 0.15;
           bcprobe(bc,14);
	}
	if(xt <= 3.913 && xt >= 3.86){
	   bc = (3.913 - xt)*((0.12 - 0.08)/(3.913 - 3.86)) - 0.12;
           bcprobe(bc,15);
	}
	if(xt <= 3.86 && xt >= 3.84){
	   bc = (3.86 - xt)*((0.08 - 0.06)/(3.86 - 3.84)) - 0.08;
           bcprobe(bc,16);
	}
	if(xt <= 3.84 && xt >= 3.815){
	   bc = (3.84 - xt)*((0.06 - 0.04)/(3.84 - 3.815)) - 0.06;
           bcprobe(bc,17);
	}
	if(xt <= 3.815 && xt >= 3.79){
	   bc = (3.815 - xt)*((0.04 - 0.05)/(3.815 - 3.79)) - 0.04;
           bcprobe(bc,18);
	}
	if(xt <= 3.79 && xt >= 3.77){
	   bc = (3.79 - xt)*((0.05 - 0.06)/(3.79 - 3.77)) - 0.05;
           bcprobe(bc,19);
	}
	if(xt <= 3.77 && xt >= 3.76){
	   bc = (3.77 - xt)*((0.06 - 0.07)/(3.77 - 3.76)) - 0.06;
           bcprobe(bc,20);
	}
	if(xt <= 3.76 && xt >= 3.74){
	   bc = (3.76 - xt)*((0.07 - 0.10)/(3.76 - 3.74)) - 0.07;
           bcprobe(bc,21);
	}
	if(xt <= 3.74 && xt >= 3.73){
	   bc = (3.74 - xt)*((0.10 - 0.15)/(3.74 - 3.73)) - 0.10;
           bcprobe(bc,22);
	}
	if(xt <= 3.73 && xt >= 3.72){
	   bc = (3.73 - xt)*((0.15 - 0.19)/(3.73 - 3.72)) - 0.15;
           bcprobe(bc,23);
	}
	if(xt <= 3.72 && xt >= 3.68){
	   bc = (3.72 - xt)*((0.19 - 0.25)/(3.72 - 3.68)) - 0.19;
           bcprobe(bc,24);
	}
	if(xt <= 3.68 && xt >= 3.64){
	   bc = (3.68 - xt)*((0.25 - 0.65)/(3.68 - 3.64)) - 0.25;
           bcprobe(bc,25);
	}
	if(xt <= 3.64 && xt >= 3.62){
	   bc = (3.64 - xt)*((0.65 - 0.9)/(3.64 - 3.62)) - 0.65;
           bcprobe(bc,26);
	}
	if(xt <= 3.62 && xt >= 3.59){
	   bc = (3.62 - xt)*((0.9 - 1.2)/(3.62 - 3.59)) - 0.9;
           bcprobe(bc,27);
	}
	if(xt <= 3.59 && xt >= 3.55){
	   bc = (3.59 - xt)*((1.2 - 1.48)/(3.59 - 3.55)) - 1.2;
           bcprobe(bc,28);
	}
	if(xt <= 3.55 && xt >= 3.54){
	   bc = (3.55 - xt)*((1.48 - 1.76)/(3.55 - 3.54)) - 1.48;
           bcprobe(bc,29);
	}
	if(xt <= 3.54 && xt >= 3.53){
	   bc = (3.54 - xt)*((1.76 - 2.03)/(3.54 - 3.53)) - 1.76;
           bcprobe(bc,30);
	}
	if(xt <= 3.53 && xt >= 3.51){
	   bc = (3.53 - xt)*((2.03 - 2.31)/(3.53 - 3.51)) - 2.03;
           bcprobe(bc,31);
	}
	if(xt <= 3.51 && xt >= 3.49){
	   bc = (3.51 - xt)*((2.31 - 2.62)/(3.51 - 3.49)) - 2.31;
           bcprobe(bc,32);
	}
	if(xt <= 3.49 && xt >= 3.42){
	   bc = (3.49 - xt)*((2.62 - 4.2)/(3.49 - 3.42)) - 2.62;
	   bcprobe(bc,33);
	}	
	return(bc);
    }
}
void bcprobe(double bc, int i) // test for
//                             non-negative B.C.
//                             and stop with position display.
{
	if(bc > 0.0){
	    printf("Error at probe[%d], bc > 0.0.",i);
            printf("Enter 1 to proceed :");
	    scanf("%s",value);  // Entry procedure more immune to
	    d = atof(value);    // invalid inputs than simple scanf
	} 
}
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .