//
//  starz5ops.cpp - Module for STARZ4 - Supplemental Ops
//
//
//    - PROJECT STARRANGER -  Programmer: James A. Green.
//	   starranger.prj is the project file
//	       producing starranger.exe from
//		     starz5.cpp - main module.
//		      starz5ag.cpp - Stellar ages and evolution, perturbations.
//		      starz5au.cpp - Neutrino luminosity & stellar wind.
//		      starz5fu.cpp - basic functions.
//		      starz5bo.cpp - bolometric corrections.
//		      starz5op.cpp - supplemental ops.
//                    starz5sc.cpp - stellar ranging.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//
//       CONTENTS
//
//       void tsurf(void) - View Tsurface from spectral type.
//       void spectraldata(void) - Display reference spectral data.
//       void listmainseq(void) - List properties of main seq. stars.
//       void brighteststars(void)  - The brightest stars, properties.
//       void neareststars(void) -  The nearest stars, properties.
//       void morehelp(void)  - More help - references.
//
//     EXTERNAL LIBRARIES
//
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//    GLOBAL VARIABLES - defined here in starz5op.cpp & globally known.
//
//				PROTOTYPE DEFINITIONS
void tsurf(void);          // - View Tsurface from spectral type.
void spectraldata(void);   // - Display reference spectral data.
void listmainseq(void);    // - List properties of main seq. stars.
void brighteststars(void); // - The brightest stars, properties.
void neareststars(void);   // -  The nearest stars, properties.
void morehelp(void);       // - More help - references.
void whitedwarfs(void);    // - Characteristics of White Dwarfs
//
//   EXTERNAL VARIABLES - defined in starz4.cpp.
//
extern int d;		   // integer string entry value
extern char value[50];     // string for entry value
/**************************************************************
*
*  tsurf() - View Tsurface from the spectral type.
*
***************************************************************/
void tsurf(void)
{
     int proceedflag = 0;

     printf("Spectral Type and Tsurface in degrees Kelvin (Zombeck)\n");
     printf("   Regular Main Sequence Stars [V]    || Giant Stars [III]\n");
     printf("Type  Temp    Type Temp   Type  Temp  || Type  Temp\n");
     printf("O5-O7 38000K  A0  10800K  K0    5240K || G5    5010K\n");
     printf("O8-O9 35000K  A2   9730K  K2    4780K || G8    4870K\n");
     printf("O9.5  31900K  A5   8620K  K5    4558K?|| K0    4720K\n");
     printf("              A7   8190K  K7    4410K || K2    4460K\n");
     printf("B0    30000K                          || K3    4210K\n");                    
     printf("B0.5  27000K  F0   7240K  M0    3920K || K4    4010K\n");
     printf("B1    24200K  F2   6930K  M1    3680K || K5    3780K\n");
     printf("B2    22100K  F5   6540K  M2    3500K || M0    3660K\n");
     printf("B3    18800K  F8   6200K  M3    3360K || M1    3600K\n");
     printf("B5    16400K              M4    3230K || M2    3500K\n");
     printf("B6    15400K  G0   5920K  M5    3120K || M3    3300K\n");
     printf("B7    14500K  G2   5780K  M6    2500K?|| M4    3100K\n");
     printf("B8    13400K  G5   5610K  M7    2380K?|| M5    2950K\n");
     printf("B9    13400K  G8   5490K  M8    2260K || M6    2800K\n");
     printf("Spectral Type Characteristics\n");
     printf("O > 25000K,          B 11000K < T < 25000K, A 7500K < T < 11000K\n");
     printf("F 6000K < T < 7500K, G 5000K < T < 6000K,   K 3500K < T < 5000K\n");
     printf("M 2200K < T < 3500K. Classes with different spectral lines are:\n");
     printf("S 2200K < T < 3500K, R(C) 5000K < T < 3500K,N(C) 3500 < T < 2200,\n");
     printf("W > 25000K,          WC > 25000K,           WN > 25000K [Wolf-Rayets]\n");
     printf("ENTER 3 -> main menu, 5 -> more spectral data:");
     scanf("%s",value);       // Entry procedure more immune to
     d = atof(value);         // invalid inputs than simple scanf
     proceedflag = d;
     if(proceedflag == 5)
         spectraldata();

     return;
}
/***********************************************
*
*  spectraldata() - Display spectral data - Reference
*
************************************************/
void spectraldata(void)
{
      int proceedflag;      

      printf("SPECTRAL DATA REFERENCE - Annie Jump Cannon\n");
      printf("COLORS:        red, orange, yellow, green,   blue,  violet,\n");
      printf("nanometers:    >610 590-610 570-590 500-570 450-500 < 450.\n");
      printf("CLASS   spectra   color      color_index  eff.temp.  examples\n");
      printf("O    He II,He I   blue        -0.3     28,000-50,000  epsilon Ori.\n");
      printf("B      He I, H    blue-white  -0.2      9,900-28,000 Rigel,Spica\n");
      printf("A         H       white        0.0      7,400-9,900  Vega,Sirius\n");
      printf("F      metals,H   yellow-white 0.3      6,000-7,400  Procyon\n");
      printf("G    Ca II,metals yellow       0.7      4,900-6,000  Sun, Alpha Cen\n");
      printf("K  Ca II, molec.  orange       1.2      3,500-4,900  Arcturus\n");
      printf("M TiO, Ca I,molec.orange-red   1.4      2,000-3,500  Betelguese\n");
      printf("R       CN,C      orange-red   1.7      3,500-5,400  ...   carbon stars\n");
      printf("S     ZrO,molec.  orange-red   1.7      2,000-3,500  R Cyg carbon stars\n");
      printf("N        C        red          > 2      1,900-3,500  R Lep carbon stars\n");
      printf("PROMINENT SPECTRAL LINES - 1 nm = 10 Angstrom Units\n");
      printf("hydrogen          -> 656.3 nm, C    neutral sodium -> 589.3 nm D\n");
      printf("neutral Helium    -> 587.6 nm, ...  neutral iron   -> 527.0 nm E\n");
      printf("neutral magnesium -> 516.7 nm,517.3, 518.4 b\n");
      printf("TiO, cool stars   -> 495.5 nm,      hydrogen       -> 486.1 nm F\n");
      printf("ionized He, hot   -> 468.6 nm,      Fe I           -> 438.4 nm d\n");
      printf("CH molecule       -> 430.0 nm, G    hydrogen H     -> 434.0 nm ...\n");
      printf("neutral Ca I      -> 422.7 nm, g    hydrogen H     -> 410.1 nm h\n");
      printf("ionized Ca II     -> 396.8 nm, H    ionized Ca II  -> 393.4 nm K\n");
      printf("Enter 3 to return to main menu :");
      scanf("%s",value);       // Entry procedure more immune to
      d = atof(value);         // invalid inputs than simple scanf
      proceedflag = d;
}
/**************************************************************
*
*   listmainseq() - List properties of well-known Main Sequence
*                   stars for use in polytrope() sketch routine
*                   above.
* 		  
*
****************************************************************/
void listmainseq(void)
{
	int select;

        printf("Properties of some well-known Main Sequence Stars..\n");
	printf("M.Schwarzschild Main Sequence chemistry X=0.75, Y=0.23, Z=0.02\n");
	printf("STAR       Derived Properties [Book] || Observed Properties || III=Giant\n");
	printf("NAME(Constella) M/Msun   R/Rsun  L/Lsun Distance mag m Spectrum->Tsurf\n");
	printf("Eltanin(Draco)                   145    100 LY   2.22   K5 III   3780K\n");
	printf("Enif(Pegasus)                    5800   780 LY   2.31   K2 Ib    4460K\n");
	printf("Fomalhaut(P.A)             2(bhm)14     23 LY    1.17   A3 V     8990K\n");
	printf("Gienah(Corvus)                   1200   450 LY   2.6    B8 III   13400K(zom)\n");
	printf("Agena(Centaur)                   10,000 490 LY   0.66   B1 II    24200K?B1 V\n");	
	printf("HELP NOTES: The internal structure of the stars is still under test.\n");
	printf("Based on theoretical analysis, we may consider stars as Eddington Standard\n");
	printf("Model polytropes with n=3 and gamma = 4/3.  These models include both the\n");
	printf("gas & radiation pressure. Lighter stars with no radiation pressure may be\n");
	printf("represented by polytropes of with n=1.5 and gamma=5/3.  As MS stars age\n");
	printf("they become more inhomogeneous, finally growing H-depleted cores in their\n");
	printf("central regions that may collapse to a white dwarf state with the ejection\n");
	printf("of a planetary nebula. Stars have lifetimes going like 13x10**9(Msun/M)**3\n");
	printf("years, so that heavier stars are more likely to be mature, with denser\n");
	printf("core regions. STARCAL considers the equation of radiative equilibrium\n");
	printf("to apply to the determination of L_radiation(r), the radiative part\n");
	printf("energy transmission partition, so that dT/dr is not determined by L(r),\n");
	printf("but by the hydrostatic equilibrium with gas plus radiation pressure\n");
	printf("as described by the n=3, gamma=4/3 polytropes. This gets good results.\n");
	printf("Enter 1->brightest stars, 2->nearest stars, 3->Main Menu, 4->Refs: ");
	scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;
	if(select == 1)
	   brighteststars();
	if(select == 2)
	   neareststars();
	if(select == 4)
	   morehelp();
	if(select == 5)
           spectraldata();

}
/*****************************************
*
*   brighteststars() - display characteristics of brightest stars
*                      for calculational input purposes.
*
******************************************/
void brighteststars(void)
{
     int select;     

     printf("THE BRIGHTEST STARS as seen from Earth.\n");
     printf("STAR              SCI.NAME   DISTANCE  VISUAL_MAG SPECTRAL_TYPE\n");
     printf("Sirius            Alpha CMa  8.6 LY    -1.46      G2 V\n");
     printf("Canopus           Alpha Car  74  LY    -0.72      A9 II\n");
     printf("Rigil Kentaurus   Alpha Cen  4.3 LY    -0.27      G2 V + K1 V\n");
     printf("Arcturus          Alpha Boo  34  LY    -0.04      K1.5 III\n");
     printf("Vega              Alpha Lyr  25  LY    -0.03      A0 V\n");
     printf("Capella           Alpha Aur  41  LY    -0.08      G6 III + G2 III\n");
     printf("Rigel             Beta Ori   1400 LY    0.12      B8 I\n");
     printf("Procyon           Alpha CMi  11.4 LY    0.38      F5 IV-V\n");
     printf("Archernar         Alpha Eri  69   LY    0.46      B3 V\n");
     printf("Betelguese        Alpha Ori  1400 LY    0.50(var) M2 I\n");
     printf("Hadar             Beta Cen   320 LY     0.61(var) B1 III\n");
     printf("Acrux             Alpha Cru  510 LY     0.76      B0.5 I + B1 V\n");
     printf("Altair            Alpha Aql  16  LY     0.77      A7 V\n");
     printf("Aldebaran         Alpha Tau  60  LY     0.85(var) K5 III\n");
     printf("Antares           Alpha Sco  520 LY     0.96(var) M1.5 I\n");
     printf("Spica             Apha Vir   220 LY     0.98(var) B1 V\n");
     printf("Pollux            Beta Gem   40 LY      1.14      K0 III\n");
     printf("Fomalhaut         Alpha PsA  22 LY      1.16      A3 V\n");
     printf("Becrux            Beta Cru   460 LY     1.25(var) B0.5 III\n");
     printf("Deneb             Alpha Cyg  1500 LY    1.25      A2 I\n");
     printf("Regulus           Alpha Leo  69  LY     1.35      B7 V\n");    
     printf("Enter 1 ->more brightest stars, 2 ->nearest stars, 3 ->main menu: ");
     scanf("%s",value);       // Entry procedure more immune to
     d = atof(value);         // invalid inputs than simple scanf
     select = d;

     if(select == 1){
	 printf("Adhara            Epsilon CMa 570 LY    1.5       B2 II\n");
         printf("Castor            Alpha Gem   49  LY    1.57      A1V + A2V\n");
         printf("Gacrux            Gamma Cru   120 LY    1.63(var) M3.5 III\n");
         printf("Shaula            Lamda Sco   330 LY    1.63(var) B1.5 IV\n");
	 printf("Enter 3 to return to main menu: ");
	 scanf("%s",value);       // Entry procedure more immune to
	 d = atof(value);         // invalid inputs than simple scanf
	 select = d;
         select = 0;
     }
     if(select == 2){
	 neareststars();
         select = 0;
     }
     if(select == 4){
	 morehelp();
	 select = 0;
     }
     if(select == 5){
	 spectraldata();
	 select = 0;
     }

}
/*****************************************
*
*   neareststars() - display characteristics of nearest stars
*                      for calculational input purposes.
*
******************************************/
void neareststars(void)
{
	int select;	

	printf("NEAREST STARS as seen from Earth.\n");
	printf("STAR              SCI.NAME   DISTANCE  VISUAL_MAG SPECTRAL_TYPE\n");
	printf("Proxima Centauri V645 Cen    4.2 LY    11.05(var)  M5.5 V\n");
	printf("Rigil Kentaurus  Alpha Cen A 4.3 LY    -0.01       G2 V\n");
	printf("                 Alpha Cen B 4.3 LY     1.33       K1 V\n");
	printf("Barnard's Star               6.0 LY     9.54       M3.8 V\n");
	printf("Wolf 359         CN Leo      7.7 LY     13.53(var) M5.8 V\n");
	printf("                 BD+36 2147  8.2 LY     7.5        M2.1 V\n");
	printf("Luyten 726-8A    UV Cet A    8.4 LY     12.52(var) M5.6 V\n");
	printf("          -8B    UV Cet B    8.4 LY     13.02(var) M5.6 V\n");
	printf("Sirus A          Alpha CMa A 8.6 LY     -1.46      A1 V\n");
	printf("      B          Alpha CMa B 8.6 LY     8.3        DA VII\n");
	printf("Ross 154                     9.4 LY     10.45      M3.6 V\n");
	printf("Ross 248                    10.4 LY     12.29      M4.9 V\n");
	printf("                Epsilon Eri 10.8 LY     3.73       K2 V\n");
	printf("Ross 128                    10.9 LY     11.10      M4.1 V\n");
	printf("                61 Cyg A    11.1 LY     5.3(var)   K3.5 V\n");
	printf("                61 Cyg B    11.1 LY     6.03       K4.7 V\n");
	printf("                Epsilon Ind 11.2 LY     4.68       K3 V\n");
	printf("                BD+43 44 A  11.2 LY     8.08       M1.3\n");
	printf("                BD+43 44 B  11.2 LY     11.06      M3.8\n");
	printf("Luyten 789-6                11.2 LY     12.18      \n");
	printf("Procyon A       Alpha CMi A 11.4 LY     0.38       F5 IV-V\n");
	printf("Enter 1->brightest stars, 2->stars & help, 3->main menu 4->Refs: ");
	scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	if( select == 1){
	   brighteststars();
	   select = 0;
	}
	if( select == 2){
	    listmainseq();
	    select=0;
	}
	if(select == 4){
	    morehelp();
	    select = 0;
	}
	if(select == 5){
	   spectraldata();
	   select = 0;
        }
}
/**********************
*
*  morehelp -  Supply references.
*
**********************/
void morehelp(void)
{
      int select;     

      printf("REFERENCES - \n");
      printf("[1] Bowers & Deeming, ASTROPHYSICS I: THE STARS, Jones & Bartlett.\n");
      printf("[2] Chiu, Hong-Yee, STELLAR PHYSICS, Ginn Blaisdell.\n");
      printf("[3] Clayton, Donald D., PRINCIPLES OF STELLAR EVOLUTION AND\n");
      printf("    NUCLEOSYNTHESIS, McGraw-Hill.\n");
      printf("[4] Green,James A., THERMONUCLEAR FUSION IN STARS, from Greenwood\n");
      printf("    Research Books & Software, Wichita, KS. See amazon.com.\n");
      printf("    Includes STARCAL,STARVIEW, STAREXP software.\n");
      printf("[5] Green, James A., CHANDRA: Chandrasekhar Sets of White\n");
      printf("    Sets of White Dwarfs & Supernova Cores, Greenwood Research.\n");
      printf("    Includes CHANDRA software for collapsed object modeling.\n"); 
      printf("[6] Phillips, A.C., THE PHYSICS OF STARS, John Wiley & Sons.\n");
      printf("[7] Rose, William K., ADVANCED STELLAR ASTROPHYSICS, Cam.Univ.Pr.\n");
      printf("[8] Schwarzschild, Martin V., STRUCTURE & EVOLUTION OF THE STARS.\n");
      printf("[9] Zombeck, M. V., HANDBOOK OF SPACE ASTRONOMY & ASTROPHYSICS.\n");
      printf("[10] SEDS: http://www.seds.org - Great Astronomy Links.\n");
      printf("[11] http://www.astro.wisc/~dolan/constellations/constellations.html\n");
      printf("DEPTHSTA is a product of Greenwood Research Books & Software.\n");
      printf("On-Line Catalog: http://grnwd.tripod.com.\n");
      printf("Contact Email: james_a_green_iii@yahoo.com. TEL:[316] 722-4954.\n");
      printf("Bookstores:      amazon.com, bn.com, and your local bookstore.\n\n\n");
      printf("Enter 1->brightest stars, 2->nearest stars, 3->Main Menu, \n");
      printf("      4->Tsurface & type, 5->spectral data, 6->White Dwarfs : ");
	scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;
	if(select == 1)
	   brighteststars();                                                \
	if(select == 2)
	   neareststars();
	if(select == 4)
	   tsurf();
	if(select == 5)
	   spectraldata();
	if(select == 6)
	   whitedwarfs();

}
/*****************************************
*
*   whitedwarfs() - display characteristics of white dwarfs
*                      for calculational input purposes.
*                   Much of the table data below is culled
*                   from BURNHAMS CELESTIAL HANDBOOK, pg.400-434
******************************************/
void whitedwarfs(void)
{
	int select;	

	printf("WHITE DWARF STARS- after Burnhams Celestial Handbook...\n");
	printf("STAR              SCI.NAME   DISTANCE  VISUAL_MAG SPECTRAL_TYPE\n");
	printf("Sirius B in Canis Major       8.7 LY    8.65      D A5\n");
	printf("  radius=30,571 km, M/Msun=0.98  \n");
	printf("Procyon B in Canis Minor      11.3 LY   10.8         ? \n");
	printf("                    M/Msun=0.65\n");
	printf("40 Eridani B in Eridanus      16 LY      9.7      D A\n");
	printf("  radius=27,353 km  M/Msun=0.44\n");
	printf("Van Maanen's Star in Pices              12.4      D G\n");
	printf("  radius=12550 km \n");
	printf("LP 357-186 in Taurus                    18.3\n");
	printf("  radius=1931 km \n");
	printf("h Per 1166                    130 LY    13.7      D A\n");
	printf("G107-70 in Lynx                30 LY    14.9      DC\n");
	printf("NGC2477-116 in Puppis         130 LY    13.7      D A\n");
	printf("HL4 in Orion, called LP658-2   22 LY    14.5      G5 or K\n");
	printf("W457 in Virgo                  42 LY    15.9      DC\n");
	printf("W489 in Virgo                  25 LY    14.7      D K\n\n\n\n");
	printf("Enter 1->brightest stars, 2->stars & help, 3->main menu 4->Refs,\n ");
	printf("     5->nearest stars    6->spectral data: ");
	scanf("%s",value);       // Entry procedure more immune to
	d = atof(value);         // invalid inputs than simple scanf
	select = d;

	if( select == 1){
	   brighteststars();
	   select = 0;
	}
	if( select == 2){
	    listmainseq();
	    select=0;
	}
	if(select == 4){
	    morehelp();
	    select = 0;
	}
	if(select == 5){
	    neareststars();
	    select = 0;
	}
	if(select == 6){
	   spectraldata();
	   select = 0;
        }
}
/*
*     References
*
* [1] - Richard Bowers & Terry Deeming, 1984. Astrophysics I: The Stars.
*       Jones & Barlett.
* [2] - Hong Yee Chiu, STELLAR PHYSICS, Ginn Blaisdell.
* [3] - Stuart L. Shapiro & Saul A. Teukolsky, 1983.
*       Black Holes, White Dwarfs, and Neutron Stars. Wiley-Interscience.
* [4] - Green, James A., CHANDRA: Chandrasekhar Sets of White Dwarfs
*           and Supernova Cores with Stellar Structure Software, 1997.
* [5] - Green, James A., THERMONUCLEAR FUSION IN STARS, Greenwood
*          Research Books & Software, 1999.
*       Shows pre-supernova stars have M/Msun > 6.23.
* [6] - A.C.Phillips, The Physics of Stars, John Wiley & Sons,pg.142-146.
* [7] - Donald D. Clayton, Principles of Stellar Evolution &
*       Nucleosynthesis, McGraw-Hill.
* [8] - Martin Schwarzschild, Structure & Evolution of the Stars,
*        Dover Pulbications, pg.9, bolometric corrections.
* [9] - Martin V. Zombeck, Handbook of Space Astronomy & Astrophysics,
*       Cambridge University Press, pp.68-69 for bolometric corrections.
* [10] - William K. Rose, Advanced Stellar Astrophysics,
*        Cambridge University Press.
* [11] - Martin V. Schwarzschild, STRUCTURE & EVOLUTION
*                                 OF THE STARS, Dover.
* [12] - E.E.Salpeter, "Nuclear Reactions in the Stars. I.
*        Proton-Proton Chain",
*        Physical Review, Volume 88, Number 3, Nov.1, 1952.
* [13] - H.A.Bethe and C.L.Critchfield, "The Formation of
*        Deuterons by Proton Combination", August 15, 1938,
*        Physical Review, pp.248-254.
*
*/
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .