#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
//
//                Programmer: James A. Green
//                Update: May 31, 2002.
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .
//
//  starz4sc.cpp -  Precision depth scanning to identify star location
//.                 from L_nuclear vs. L_observed.
//
//
//  PROJECT STARRANGER -
//	   starranger.prj is the project file
//	       producing starranger.exe from
//		     starz5.cpp - main module.
//		      starz5ag.cpp - Stellar ages and evolution, perturbations.
//		      starz5au.cpp - Neutrino luminosity & stellar wind.
//		      starz5fu.cpp - basic functions.
//		      starz5bo.cpp - bolometric corrections.
//		      starz5op.cpp - supplemental ops.
//                    starz5sc.cpp - depth scanner.
//
//               Greenwood Research, http://grnwd.tripod.com.
//
//  ROUTINES in starz5sc.cpp:
//
//   depthscanner() -  Use L_nuclear vs L_observed in Star Age calculations
//                     to find the Zero Age Main Sequence distance
//                     at which L_nuclear = L_observed, and the distances
//                     such that L_nuclear > L_observed, normalized via
//                     core depletion of hydrogen or other nuclear fuels
//                     such that the lifetime is less than the Schwarzschild
//                     Age  (13x10**9)*(M_sun/M)**3 years.
//
//
//
//
//   EXTERNAL VARIABLES
//
extern FILE *fp;                         // file pointer fp to starman.txt
extern int status;                       // file status
//extern double cuberoot(double v);        // cuberoot(v)
//extern double nthpower(double v, int n); // v**n
//extern double pi;		   //  pi, 3.14159.
//extern double gnewton;		   // Newtons constant of gravitation
extern double msun;                // mass of the Sun in kg.
extern double xmstar[1];           // mass of the star in kg.
extern double ageschw;             // Schwarzschild star age.
//extern double age;                 // Greens Main Sequence Lifetime.
//extern double hkage;               // Helmholtz contraction time.

extern double claynormx;           // X(H) to normalize L_nuclear_Clayton to L(R) observed.
extern double chemx[1];            // X(H) - initial hydrogen abundance by weight.
extern double chemy[1];            // Y(He) - initial helium abundance by weight.
extern double chemz[1];            // Z(hvy) - initial heavy element abundance by weight.
extern double chemmu[1];                 // initial molecular weight.
extern double xtc[1];                    // initial central temperature.
extern double rocoremin;                 // minimum density at far edge of H-burning core.
extern double xdensitycentral[1];        // max density at center of H-burning core.
extern double mh;                        // mass of the hydrogen atom in kg.
//extern double rhcore;			 // radius of the core in solar radii.
extern double rsun;                      // radius of the Sun in meters.
extern double totalluminosity;          // total nuclear luminosity P-P, CNO, He-burning
extern double totalhluminosity;         // total P-P plus CNO H-burning.
extern double bclratio[1];              // L_star/L_sun
extern double lsun;                     // Luminosity of the Sun in Watts
extern double xrstar[1];                // Radius of the Sun in meters.
//extern double photontime;               // The stellar photon random-walk diffusion time.
extern double lightyears[1];            // Original Distance in lightyears for depthscanner.
extern void probe(int i);               // Probe identifies self as ith, waits for keystroke.
extern double visualmag[1];             // visual magnitude
extern double surfacetemp[1];             // surface temperature
extern double bolometric(double t);       // bolometric correction routine
extern double rawabsmag[1];             // raw absolute magntiude prior to B.C.
extern double bcabsmag[1];             // bolometric-corrected abs. mag.
extern double bolo[1];             //
extern double rawlratio[1];             //
extern double xavgdensity[1];             //
extern double starform[1];             //
extern double xpressurecentral[1];             //
extern double k;             // boltzmanns constant
extern double a;
extern double xfbeta[1];             //
extern double findbeta(double fbeta);
extern double xbeta[1];             //
extern double xpgas[1];             //
extern double xpradiation[1];             //
extern double xpratio[1];             //
extern double xomega[1];             //
extern void nuclearluminosity(int j);

extern double bcmratiobest[1];             //
extern double rawmratiobest[1];             //
extern double bcrratio[1];             //
extern double rawrratio[1];             //

extern double scclaytonluminosity[1];             //
extern double normalizeclayton;
//
//   depthscanner() -  Use L_nuclear vs L_observed in Star Age calculations
//                     to find the Zero Age Main Sequence distance
//                     at which L_nuclear = L_observed, and the distances
//                     such that L_nuclear > L_observed, normalized via
//                     core depletion of hydrogen or other nuclear fuels
//                     such that the lifetime is less than the Schwarzschild
//                     Age  (13x10**9)*(M_sun/M)**3 years.
//
void depthscanner(void)
{
    int j;
    int sustain;
     double r;		// distance to the star in LY.
     double rpc;        // distance to the star in parsecs.
     double m;          // visual magnitude m.
     double absmag;     // absolute magnitude M
//                         with bolometric corrections.
     double absmagnbc;  // absolute magnitude M without
//                          bolometric corrections.
     double lratio;     // L/Lsun luminosity.
     double lrationbc;  // L/Lsun luminosity without
//                         bolometric corrections.
     double expo;       // exponent, L/Lsun = 10**exponent.
     double exponbc;    // exponent, L/Lsun = 10**exponent, w/o BC.
     double mratio;     // M/Msun mass ratio.
     double mrationbc;  // M/Msun mass ratio without
//                         bolometric corrections.
     double mratiolow;  // M/Msun mass ratio.
     double mratiolownbc; // M/Msun mass ratio w/o B.C.
     double mratiobest; // M/Msun for x = 3.409 best fit
//                         to mass-luminosity data.
     double mratiobestnbc;  // M/Msun best fit with x=3.409
//                             w/o bolometric corrections.
     double rratio;     // R/Rsun.
     double rrationbc;  // R/Rsun without bolometric corrections.
     double x = 3.0;    // L/Lsun = (M/Msun)**x.   3 <= x < 4.
     double tsurface;   // suface temperature of star for BC,
//                         bolometric correction.
     double bc;	// M.Schwarzschild bolometric correction, pg. 9. [8].
     double xt;		// xt = log10(Tsurface).
     double rstar;	// stellar radius in meters.
     
     double mstar;      // mass of the star in kg.
     
     double avgdensity; // average density of the star.
     double densitycentral; // central density of the star.
     double pressurecentral; // central pressure of the star.
     double factor;
     double fbeta;       // (1-b)/b**4 = (P**3/ro**4)*[(mu*mh)**4](a/3).
     double mu;          // molecular weight chemmu[j].
 
 
     double beta;       // b.
     double pgas;       // gas pressure.
     double pradiation; // radiation pressure.
     double pratio;     // Pradiation/Pgas
     double tc;        // central temperature
//                        tc=(mu*mh/(k*density_c))*beta*Ptotal_c
     double omega;     // binding energy of n=3
//                        polytropic model star.
     double centradpress; // central radiation
//                           pressure, gamma = 5/3 model.
     double age; // main sequence lifetime,
//                  see THERMONUCLEAR FUSION IN STARS, pg.232.
     double ageschw; // main sequence lifetime,
//                      1/M**2.409 formulae with Schw. sun age.
     double distance[30]; // distances in light-years.
     double raylum[30];   // L_nuclear/L_star
     double fmodels;      // floating point number of models
     int models;       // integer number.
     double fj;           // floating point model number.
     double deltad;       // depth step in light-years.
 
    fprintf(fp,"\nL_nuclear DEPTH SCAN \n");
    j = 0;
    models = 20;                    // Scan plus and minus 5%
    deltad = lightyears[0]/200.0;   // 0.5% depth scan steps -> 0.5% radius mods
    fmodels = models;
    while(j < models){
	fj = j;     //  use fj for real value of j.	
	x = 3.0;
	fprintf(fp,"DepthModel = %d. ",j);
	fprintf(fp,"Surface Temperature = %e. \n", surfacetemp[0]);
	r = lightyears[0]; // get distance in light-years.
	r = (r - (fj*deltad)) + (fmodels*(deltad/2.0));
        m = visualmag[0]; // get visual magnitude m.	
	tsurface = surfacetemp[0];
	xt = log10(tsurface);  // find the look-up value for BCstellar.
	bc = bolometric(xt);  // interpolate B.C.
//                               from a table as a function of xt.
//
//  Derivations start from
//     (L1/L2)(r2/r1)**2 = I1/I2 = 10**[(2/5)(m2 - m1)],
//
//  so that  m2 = m1 + (5/2)*log10(L1/L2) + 5*log10(r2/r1).
//
//  Uncorrected, L/Lsun = 10**((2/5)*(4.83 - M_v),
//      where 4.83 is the uncorrected
//      absolute visual magnitude of the Sun.
//  M_bol = M_v + B.C.  After Zombeck,[9], page 25.
//                     HANDBOOK OF SPACE ASTRONOMY & ASTROPHYSICS.
//  Corrected,  L/Lsun = 10**((2/5)*(4.83 - M_bol),
//                 where 4.83 is the uncorrected
//                 absolute magnitude of the Sun.
//  For the Sun, M_v = +4.83 (visual magnitude at 10 pc)
//               m_v = -26.74 (visual magnitude at 1 AU)
//               M_bol = +4.75 = M_v + B.C.  absolute mag
//                                          at 10 pc, corrected for
//                                           atmospheric absorption.
//               m_bol = -26.82  visual magnitude of the sun above
//                               the atmosphere at 1 AU.
//
//  Note that our ideal type G2-star is slightly hotter
//             at 5780 deg K. than the sun,
//    which has a surface temp of 5770 deg.K.
//
//  Lsun_corrected/Lsun_uncorrected = 10**[(-2/5)*B.C.]
//                                  = exp[-0.921034*B.C.]
//                                  = exp[-2.3025851*(2/5)*B.C.].
//
//  (L/Lsun)_both_corrected
//        = 10**[(2/5)(Mvsun + B.C.sun - (Mv + B.C.star))]
//        = 10**[(2/5)(M_bol_sun - M_bol)].
//
// We must use L/Lsun_corrected, whether or not L
//     is corrected or anomolies result.
//
//   L/Lsun = 10**((2/5)(M_bol - M_v)) x 10**(-2/5)(B.C.),
//            second term prepares after B.C.
//
// Note L/Lsun
// = [(r/r_sun)**2]*[10**(2/5)(m_sun_bol - m_v)] x 10**(-2/5)B.C.,
//
// where the 2nd term gives the effect of introducing
// the bolometric correction to m_v.
//
	rpc = r/3.2600;  // compute distance in parsecs.
	absmag = m + 5.0 - (5.0*log10(rpc));   // cal absolute
//                                                magnitude M.
	absmagnbc = absmag;       // save pre-corrected w/o BC.
        rawabsmag[0] = absmag;   // save to report variable.
	absmag = absmag + bc;    // include stellar bolometric BC.
	bcabsmag[0] = absmag;    // save to report variable.
        bolo[0] = bc;            // save B.C.
	expo = (4.83 - 0.068824 - absmag)/2.5; // Must use program's
//                                      4.83 + B.C.= 4.83 - 0.068072
//         m_bol_sun     or use 0.068824 for Sun at exact temp 5770.0
//         4.761176      for calibration, because our G2 is not
//                       precisely the same as the Sun, which is
//                      just a little cooler.  We must have the
//                      ratio for both corrected for astro research.
//                     Based on Zombeck's table of corrections,
//  m_bol_sun=4.761176        M_bol_sun = 4.83 - 0.068824 = 4.761176
	exponbc = (4.83 - 0.068824 - absmagnbc)/2.5;
//                                     exponent without bolometric corr.
	lratio = exp(2.302585*expo); // e**Ax = 10**x, A = 2.302585
        bclratio[0] = lratio;   // save to report variable.
	lrationbc = exp(2.302585*exponbc); // same for no B.C.
        rawlratio[0] = lrationbc;
	fprintf(fp,"L/Lsun = %e, with bolometric correction BC = %f\n",
	lratio,bc);	
	if(lratio > 1.0e7){
	   fprintf(fp,"!Error!\nL/Lsun = %e >> Max = 10**6 for stable physical stars.\n",
	   lratio);
	   fprintf(fp,"Probable cause: parameter entry invalid.\n");
	   fprintf(fp,"  Distance = %e LY\n",r);
	   fprintf(fp,"  Visual magnitude = %f \n",m);
	   fprintf(fp,"May cause numeric exception! -> EXIT DEPTHSTA.\n");
           exit(0);
	}
//
// L/Lsun = (M/Msun)**x, where x = about 3, so M/Msun = (L/Lsun)**(1/x).
// Then log10(M/Msun) = (1/x)log10[L/Lsun],
// so M/Msun = 10**[(1/x)*log10[L/Lsun] = exp[(2.302585/x)*log10[L/Lsun].
//
	mratio = exp((2.302585/x)*log10(lratio));
        mrationbc = exp((2.302585/x)*log10(lrationbc));
	x = 4.0;
	mratiolow =  exp((2.302585/x)*log10(lratio));
	mratiolownbc = exp((2.302585/x)*log10(lrationbc));
//                                            again w/o B.C.	
	x = 3.409; // empirical best guess from chart measurements.
	mratiobest =  exp((2.302585/x)*log10(lratio));
        bcmratiobest[0] = mratiobest;
	mratiobestnbc =  exp((2.302585/x)*log10(lrationbc));
	rawmratiobest[0] = mratiobestnbc;

	fprintf(fp,"M/Msun = %f , x = %f  best mass-lum fit with B.C..\n",
	mratiobest, x);	
//
//  To do nuclear luminosity boundaries, re-run
//   nuclearluminosity with mratiolow & mratio.
//              to pick the central density and mass.
//
// Mass-Luminosity Chart:
//   See THERMONUCLEAR FUSION IN STARS, by James A.Green,
//   Figure 1.25, page 34.
        
	rratio = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lratio);
	
        bcrratio[0] = rratio;
	fprintf(fp,"R/Rsun = %e with bolometric corrections.\n",
	rratio);
        
	rrationbc = (5770.0/tsurface)*(5770.0/tsurface)*sqrt(lrationbc);
	rawrratio[0] = rrationbc;	


	fprintf(fp,"Visual magnitude   m = %f,", m);
	age = 9.92e9*(1.0/mratiobest)*(1.0/mratiobest);
//                                              t0(Msun/M)**2 lifetime.
//
//  t_ms = delta_X*M*Ecc/L,
//  t_sun = delta_X*M*Ecc/Lsun,
//          with delta_X = 0.13 (Schwarzschild)
//  Therefore,
//   t_ms = t_sun(M/Msun)(Lsun/L) = t_sun(Msun/M)**2.409.
//                                = t_sun*exp(2.409*log(Msun/M)).
//
	ageschw = 12.88e9*exp( 2.409*log( 1.0/mratiobest ));
//                      tsun_Schw*(Msun/M)**2.409 lifetime.
	fprintf(fp,"  MS Lifetime = %e years.\n",ageschw);

	fprintf(fp,"Absolute magnitude with bolometric cor. M + B.C. = %f\n",
	absmag);	
	fprintf(fp,"r = %e light-years = %e parsecs.\n", r, rpc);
	distance[j] = r; //  light-years
//
// Compute the average and the central density for n=3 polytrope.
//    
	     rstar = rratio*rsun;
	     xrstar[0] = rstar;    // save in report
//                                    radius of star in meters.
	     mstar = mratiobest*msun;
	     xmstar[0] = mstar;   // save in report mass of star in kg.

	     fprintf(fp,"Radius = %e meters. ",rstar);
	     fprintf(fp,"Mass = %e kg.\n", mstar);
     
	     avgdensity = mstar/((4.0/3.0)*3.14159*(rstar*rstar*rstar));
	     xavgdensity[0] = avgdensity;
	     
             if(starform[0] == 0){   // gamma=4/3 central density
	         densitycentral = xavgdensity[0]*54.1825;
		 xdensitycentral[0] = densitycentral;		 
	     }
	     else if(starform[0] == 1){
		 densitycentral = avgdensity*5.99071;
//                                     see pg.84, THERMONUCLEAR
		 xdensitycentral[0] = densitycentral;
//                                      FUSION IN STARS, by J.A.Green
	     }
	     else if(starform[0] == 2){
	         densitycentral = avgdensity*152.884; // see p.427, Chiu,
		 xdensitycentral[0] = densitycentral; // STELLAR PHYSICS.
	     }
	     else if(starform[0] == 3){
		 densitycentral = avgdensity*622.408; // see p.427, Chiu,
		 xdensitycentral[0] = densitycentral; // STELLAR PHYSICS.
             }
//
// Compute the central total pressure for polytrope of index n
//             = (GM**2/R**4)*[(n+1)4pi(dtheta/dx)**2)]**-1

	     pressurecentral =   mstar*mstar*6.67e-11/(rstar*rstar*rstar*rstar);
             if(starform[0] == 0){       // gamma=4/3, n=3 model
		 factor = 4.0*3.14159*4.0*(0.04231)*(0.04231);
	     }
	     else if(starform[0] == 1){ // gamma=5/3, n=1.5 model
		 factor = 2.5*3.14159*4.0*(2.71406/(3.65375*3.65375))*(2.71406/(3.65375*3.6375));
	     }
	     else if(starform[0] == 2){ // gamma=9/7 n=3.5 model
		 factor = 4.5*3.14159*4.0*(1.89056/(9.53581*9.53581))*(1.89056/(9.53581*9.53581));
	     }
	     else if(starform[0] == 3){ //  gamma=5/4, n=4.0 model
		 factor = 5.0*3.14159*4.0*(1.79723/(14.97155*14.97155))*(1.79723/(14.97155*14.97155));
	     }

	     factor = 1.0/factor;
	     pressurecentral = pressurecentral*factor;
             xpressurecentral[0] = pressurecentral;

	     fprintf(fp,"Average density=%e kg/m**3. ",
	     avgdensity);
	     fprintf(fp,"Central density=%e kg/m**3.\n",
	     densitycentral);
	     fprintf(fp,"Central total pressure = %e Newtons/m**2.\n",
	     pressurecentral);

//
// For gamma = 4/3 models,
// find (1 - beta)/beta**4 = ( (Pc**3)/ro_c**4 )*[(mu*mh/k)**4]*(a/3).
//
	     mu = chemmu[0];
	     fbeta = pressurecentral*pressurecentral*pressurecentral;

	     fbeta = fbeta/(densitycentral*densitycentral*densitycentral*densitycentral);

	     fbeta = fbeta*(mu*mh/k)*(mu*mh/k)*(mu*mh/k)*(mu*mh/k);
	     fbeta = (a/3.0)*fbeta;
	     xfbeta[0] = fbeta; // report model variable.


             if(starform[0] == 0)
		fprintf(fp,"[1 - beta]/beta**4 = %e. ", fbeta);

	     if( mratiobest > 2.0){
		fprintf(fp,"Carbon Cycle ON: M > 2*M_sun\n");  // Test for carbon cycle on.
	     }
	     else{
		fprintf(fp,"\n");
	     }     	    


	     beta = findbeta(fbeta);
             if(starform[0] == 0){
		  xbeta[0] = beta;
	     }
	     else if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){
		  xbeta[0] = 1.0;
                  beta = 1.0;       // radiation pressure neglected.
	     }
             if(starform[0] == 0) // if gamma=4/3
		  fprintf(fp,"Beta = %e. Pgas = beta*Ptotal, Pradiation = (1-beta)*Ptotal.\n",
		   beta);
	     
	     pgas = beta*pressurecentral;
             xpgas[0] = pgas;
	     pradiation = (1.0 - beta)*pressurecentral;
             xpradiation[0] = pradiation;
	     pratio = pradiation/pgas;
	     xpratio[0] = pratio;

             if(starform[0] == 0){
	         fprintf(fp,"Pradiation/Ptotal = %e. ", 1.0 - beta);
	         fprintf(fp,"Pgas/Ptotal = %e.\n", beta);
	         fprintf(fp,"Pradiation/Pgas = %e. ", pratio );
             }
             if(starform[0] == 0){      // gamma = 4/3, n = 3 model
		  tc = (mu*mh/(k*densitycentral))*beta*pressurecentral;
             }
	     else if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){  
                  tc = pressurecentral*mu*mh/(k*densitycentral);
             }
             xtc[0] = tc;
	     fprintf(fp,"Tcentral = %e deg Kelvin.\n",tc);
	     if(starform[0] == 1 || starform[0] == 2 || starform[0] == 3){
                centradpress = (1.0/3.0)*a*tc*tc*tc*tc;
		fprintf(fp,"Central Radiation pressure = %e Newtons/m**2.\n",
		centradpress);
	     }

             if(starform[0] == 0)
		  omega = (3.0/(5.0 - 3.0))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 1)
		  omega = (3.0/(5.0 - 1.5))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 2)
		  omega = (3.0/(5.0 - 3.5))*(6.67e-11)*mstar*mstar/(rstar);
	     else if(starform[0] == 3)
		  omega = (3.0/(5.0 - 4.0))*(6.67e-11)*mstar*mstar/(rstar);
	     xomega[0] = omega;

	     fprintf(fp,"Grav binding energy = %e J. \n",omega);

	     if(bcmratiobest[0] > 6.23){
		 fprintf(fp,"Pre-Supernova M/Msun > 6.23.\n");
	     }
	     else{
		 fprintf(fp,"Not Pre-Supernova: M/Msun < 6.23.\n");
	     }
	     probe(1);
//
//    Pre-renormalize the normalization constants so stellar age calc can proceed
//       for any choice of the initial abundances of the elements.
//
	     nuclearluminosity(0); // Pre-compute  normalizeclayton
//				      for our specific present abundances X(i) = X,Y,...
	     probe(2);
	     normalizeclayton = scclaytonluminosity[0];  // screened clayton to pre-renormalize
//
	     nuclearluminosity(0);  // Compute the nuclear
//               			       luminosity of the star,
	     
	     fprintf(fp,"P-P chain Lnuclear/Lstar=%e. ",scclaytonluminosity[0]/lratio);
	     fprintf(fp,"Clayton screened & unnormalized.\n");

	     fprintf(fp,"P-P + CNO + He-burning Lnuclear/Lstar = %e. \n", totalluminosity/lratio);
	     raylum[j] = totalluminosity/lratio;

// end of calc with bolometric corrections.
	j++; // on to next model in depth scan sequence of models.

    }
    j = 0;
    while(j < models){   // print out depth scan models.
       fprintf(fp,"%d, Lnuclear/Lstar=%e, distance=%e LY\n", j, raylum[j], distance[j]);
       j++;
    }

}
//
//
//
//    - PROJECT STARMAN -  Programmer: James A. Green.
//	   starman.prj is the project file
//	       producing starman.exe from
//		     starz4.cpp - main module.
//		      starz4ag.cpp - Stellar ages and evolution, perturbations.
//		      starz4au.cpp - Neutrino luminosity & stellar wind.
//		      starz4fu.cpp - basic functions.
//		      starz4bo.cpp - bolometric corrections.
//		      starz4op.cpp - supplemental ops.
//
//
//  	          Email: james_a_green_iii@yahoo.com
//                Online: http://grnwd.tripod.com/home.html .
//  	          Greenwood Research Catalog: http://grnwd.tripod.com .